import processing.opengl.*;
import peasy.*;
import controlP5.*;

PeasyCam cam;
ControlP5 controlP5;
RadioButton r;
MyControlListener myListener;
PMatrix3D currCameraMatrix;
PGraphics3D g3;
PGraphics txt;
PFont myFont;

float deg = 0.0;
float radius = 250.0;
float radiusMini = 100.0;

int whichYear = 0;
int thisYear = 0;
int row = 3482;
int col = 4;
int day = 0;
float scaleIt = 50.0;
float scaleItHud = 125.0;

float[] rotations = new float[3]; 

String[] whichDewey = {"[000]", "[100]", "[200]", "[300]", "[400]", "[500]", "[600]", "[700]", "[800]", "[900]"};
String[] thisDewey = {"Computer Science", "Philosophy", "Religion", "Social Science", "Language", "Science", "Technology & Applied Science", "Arts & Recreation", "Literature", "History & Geography",};

int[][] spl = new int [row][col];
int[][] blank = new int [row][col];
int[][] spl2009 = new int [row][col];
int[][] spl2010 = new int [row][col];
int[][] spl2011 = new int [row][col];


void setup() {
  smooth();
  size(1280, 720, OPENGL);
  frameRate(30);
  colorMode(HSB, 360, 100, 100);
  myFont = loadFont("Standard0756-48.vlw"); 
  textFont(myFont);
  g3 = (PGraphics3D)g;

  //Control setup
  controlP5 = new ControlP5(this);
  controlP5.addSlider("day", 0, 347, 0, 50, height - 50, width - 125, 20);
  controlP5.setAutoDraw(false);
  myListener = new MyControlListener();
  controlP5.controller("day").addListener(myListener);
  r = controlP5.addRadioButton("radioButton", 50, 50);
  
  controlP5.setColorLabel(color(30, 30, 32));  
  controlP5.setColorBackground(color(30, 30, 100));
  controlP5.setColorForeground(color(30, 30, 32));
  controlP5.setColorActive(color(30, 30, 65));
  
  r.setColorActive(color(192, 47, 23));
  r.setColorLabel(color(0, 0, 100));
  r.setItemsPerRow(3);
  r.setSpacingColumn(50);
  //add radio buttons
  addToRadioButton(r, "2009", 1);
  addToRadioButton(r, "2010", 2);
  addToRadioButton(r, "2011", 3);

  //Camera setup
  cam = new PeasyCam(this, width/2, height/2, 0, 800);
  cam.setMinimumDistance(10);
  cam.setMaximumDistance(2500);

  //Parse SPL 2005 - 2011 data  
  String splData0[] = loadStrings("spl2011.txt");
  String splData1[] = loadStrings("spl2010.txt");
  String splData2[] = loadStrings("spl2009.txt");


  for (int i = 1; i < splData0.length; i++) {   
    String splTemp0[] = splitTokens(splData0[i], ",");
    for (int j = 0; j < col; j++) {
      spl2011[i][j] = Integer.parseInt(splTemp0[j]);
    }
  }
  
  for (int i = 1; i < splData1.length; i++) {   
    String splTemp1[] = splitTokens(splData1[i], ",");
    for (int j = 0; j < col; j++) {
      spl2010[i][j] = Integer.parseInt(splTemp1[j]);
    }
  }

  for (int i = 1; i < splData2.length; i++) {   
    String splTemp2[] = splitTokens(splData2[i], ",");
    for (int j = 0; j < col; j++) {
      spl2009[i][j] = Integer.parseInt(splTemp2[j]);
    }
  }  
}  

void draw() {
  background(152, 7, 98);
  cam.setMouseControlled(true);
  rotations = cam.getRotations(); // get rotations
  
  if (whichYear == 0){
    spl = blank;
    thisYear = 0;
  } else if (whichYear == 1){
    spl = spl2009;    
    thisYear = 2009;  
  } else if (whichYear == 2){
    spl = spl2010;    
    thisYear = 2010;
  } else if (whichYear == 3){
    spl = spl2011;    
    thisYear = 2011;
  } 
  
  drawGraphics();  

  noLights();
  cam.beginHUD();
  stroke(30, 30, 32);
  fill(30, 30, 32);

  for (float i = 0.0; i < 360.0; i = i + 36) {
    textSize(8);
    float angle = radians(deg);
    float baseXmini = (width - 175) + (cos(angle) * radiusMini);
    float baseYmini = 175 + (sin(angle) * radiusMini);
    float baseXminiText = (width - 175) + (cos(angle) * (radiusMini + 15));
    float baseYminiText = 175 + (sin(angle) * (radiusMini + 15));    
    textAlign(CENTER);
    text(whichDewey[int(i/36)], baseXminiText, baseYminiText);
    strokeWeight(1);
    line(baseXmini, baseYmini, 0, width - 175, 150 + 25, 0);
    textAlign(RIGHT);
    textSize(8);
    text(whichDewey[int(i/36)], width - 200, 350 + (i - (11 * int(i/36))) + 25);
    text(thisDewey[int(i/36)], width - 235, 350 + (i - (11 * int(i/36))) + 25);

    deg = i;
  }
  
  text("[dewey class]", width - 200, 350);
  textAlign(LEFT);
  text("[count]", width - 150, 350);
  text("[ % diff ]", width - 100, 350);
  text("Select year", 50, 45);
  text("Scroll through days", 50, height - 55);
  
  //draw % difference radar map
  noStroke();
  fill(352, 73, 83, 180); 
  for (int i = 1; i < spl.length - 20; i = i + 10){
  if (((i + 10) - (day * 10)) != 1){
    } else {
  beginShape();  
  vertex(((percentDiff(spl[i][3], spl[i + 10][3]))/2.5) + (width - 175), (-0.72654253 * ((percentDiff(spl[i][3], spl[i + 10][3]))/2.5)) + 175);
  vertex(((percentDiff(spl[i+1][3], spl[(i+1) + 10][3]))/2.5) + (width - 175), 175);
  vertex(((percentDiff(spl[i+2][3], spl[(i+2) + 10][3]))/2.5) + (width - 175), (0.7265424 * ((percentDiff(spl[i+2][3], spl[(i+2) + 10][3]))/2.5)) + 175);
  vertex(((percentDiff(spl[i+3][3], spl[(i+3) + 10][3]))/5) + (width - 175), (3.0776842 * ((percentDiff(spl[i+3][3], spl[(i+3) + 10][3]))/5)) + 175);
  vertex((((percentDiff(spl[i+4][3], spl[(i+4) + 10][3]))/5) * -1) + (width - 175), (3.0776842 * ((percentDiff(spl[i+4][3], spl[(i+4) + 10][3]))/5)) + 175);
  vertex((((percentDiff(spl[i+5][3], spl[(i+5) + 10][3]))/2.5) * -1) + (width - 175), (0.7265424 * ((percentDiff(spl[i+5][3], spl[(i+5) + 10][3]))/2.5)) + 175);
  vertex((((percentDiff(spl[i+6][3], spl[(i+6) + 10][3]))/2.5) * -1) + (width - 175), 175);
  vertex((((percentDiff(spl[i+7][3], spl[(i+7) + 10][3]))/2.5) * -1) + (width - 175), (-0.7265424 * ((percentDiff(spl[i+7][3], spl[(i+7) + 10][3]))/2.5)) + 175);
  vertex((((percentDiff(spl[i+8][3], spl[(i+8) + 10][3]))/5) * -1) + (width - 175), (-3.0776842 * ((percentDiff(spl[i+8][3], spl[(i+8) + 10][3]))/5)) + 175);
  vertex(((percentDiff(spl[i+9][3], spl[(i+9) + 10][3]))/5) + (width - 175), (-3.0776842 * ((percentDiff(spl[i+9][3], spl[(i+9) + 10][3]))/5)) + 175);
  endShape(CLOSE); 
  
  textSize(8);
  textAlign(LEFT);
  fill(352, 73, 83, 180);
  //Print % diff
  text("[ " + percentDiff(spl[i][3], spl[i + 10][3]) + "% ]", width - 100, 350 + 25);
  text("[ " + percentDiff(spl[i+1][3], spl[(i+1) + 10][3]) + "% ]", width - 100, 375 + 25);  
  text("[ " + percentDiff(spl[i+2][3], spl[(i+2) + 10][3]) + "% ]", width - 100, 400 + 25);  
  text("[ " + percentDiff(spl[i+3][3], spl[(i+3) + 10][3]) + "% ]", width - 100, 425 + 25);  
  text("[ " + percentDiff(spl[i+4][3], spl[(i+4) + 10][3]) + "% ]", width - 100, 450 + 25);  
  text("[ " + percentDiff(spl[i+5][3], spl[(i+5) + 10][3]) + "% ]", width - 100, 475 + 25);  
  text("[ " + percentDiff(spl[i+6][3], spl[(i+6) + 10][3]) + "% ]", width - 100, 500 + 25);  
  text("[ " + percentDiff(spl[i+7][3], spl[(i+7) + 10][3]) + "% ]", width - 100, 525 + 25);  
  text("[ " + percentDiff(spl[i+8][3], spl[(i+8) + 10][3]) + "% ]", width - 100, 550 + 25);  
  text("[ " + percentDiff(spl[i+9][3], spl[(i+9) + 10][3]) + "% ]", width - 100, 575 + 25); 
    }
  }
  
  //Draw count radar map
  for (int i = 1; i < spl.length - 10; i = i + 10){
    if ((i - (day * 10)) != 1){
    } else {
  fill(192, 47, 23, 180);
  noStroke();
  beginShape();
  vertex((spl[i][3]/scaleItHud) + (width - 175), (-0.72654253 * (spl[i][3]/scaleItHud)) + 175);
  vertex((spl[i+1][3]/scaleItHud) + (width - 175), 175);
  vertex((spl[i+2][3]/scaleItHud) + (width - 175), (0.7265424 * (spl[i+2][3]/scaleItHud)) + 175);
  vertex((spl[i+3][3]/(scaleItHud*2)) + (width - 175), (3.0776842 * (spl[i+3][3])/(scaleItHud*2)) + 175);
  vertex(((spl[i+4][3]/(scaleItHud*2)) * -1) + (width - 175), (3.0776842 * (spl[i+4][3]/(scaleItHud*2))) + 175);
  vertex(((spl[i+5][3]/scaleItHud) * -1) + (width - 175), (0.7265424 * (spl[i+5][3]/scaleItHud)) + 175);
  vertex(((spl[i+6][3]/scaleItHud) * -1) + (width - 175), 175);
  vertex(((spl[i+7][3]/scaleItHud) * -1) + (width - 175), (-0.7265424 * (spl[i+7][3]/scaleItHud)) + 175);
  vertex(((spl[i+8][3]/(scaleItHud*2)) * -1) + (width - 175), (-3.0776842 * (spl[i+8][3]/(scaleItHud*2))) + 175);
  vertex((spl[i+9][3]/(scaleItHud*2)) + (width - 175), (-3.0776842 * (spl[i+9][3]/(scaleItHud*2))) + 175);
  endShape(CLOSE);
  
  textSize(8);
  textAlign(LEFT);
  fill(30, 30, 32);
  //Print count
  text(spl[i][3], width - 150, 350 + 25);
  text(spl[i+1][3], width - 150, 375 + 25);  
  text(spl[i+2][3], width - 150, 400 + 25);  
  text(spl[i+3][3], width - 150, 425 + 25);  
  text(spl[i+4][3], width - 150, 450 + 25);  
  text(spl[i+5][3], width - 150, 475 + 25);  
  text(spl[i+6][3], width - 150, 500 + 25);  
  text(spl[i+7][3], width - 150, 525 + 25);  
  text(spl[i+8][3], width - 150, 550 + 25);  
  text(spl[i+9][3], width - 150, 575 + 25);    
  //Print Month / Day
  text("year [ " + thisYear + " ]", 50, 100);
  text("month [ " + spl[i][0] + " ]", 50, 125);
  text("day [ " + spl[i][1] + " ]", 50, 150);
    }
  }
    
  strokeWeight(1);
  stroke(30, 30, 32);
  line(width - 175, 375 - 6, width - 175, 600);
  cam.endHUD();
}

void drawGraphics() { 
  //Lines for labels 
  stroke(30, 30, 32);
  fill(30, 30, 32);
  textSize(20);
  for (float i = 0.0; i < 360.0; i = i + 36) {
    float angle = radians(deg);
    float baseX = width/2 + (cos(angle) * radius);
    float baseY = height/2 + (sin(angle) * radius);
    float baseXtext = width/2 + (cos(angle) * (radius + 50));
    float baseYtext = height/2 + (sin(angle) * (radius + 50));
    textAlign(CENTER);
    strokeWeight(2);
    line(baseX, baseY, 0, width/2, height/2, 0);
    
    pushMatrix();
    translate(baseXtext, baseYtext);
    rotateX(rotations[0]);
    rotateY(rotations[1]);
    rotateZ(rotations[2]);
    
    text(whichDewey[int(i/36)], 0, 0);
    popMatrix();
    
    deg = i;

    //println(tan(angle));
  }
  
  strokeWeight(1);
  for (int i = 1; i < spl.length - 10; i = i + 10){
  if ((i - (day * 10)) != 1){
      noFill();
      stroke(192, 47, 23, 100);
  } else {
      noStroke();
      fill(192, 47, 23, 180);
  }
  //draw actual count
  beginShape();  
  vertex((spl[i][3]/scaleIt) + width/2, (-0.72654253 * (spl[i][3]/scaleIt)) + height/2, i - (day * 10));
  vertex((spl[i+1][3]/scaleIt) + width/2, height/2, i - (day * 10));
  vertex((spl[i+2][3]/scaleIt) + width/2, (0.7265424 * (spl[i+2][3]/scaleIt)) + height/2, i - (day * 10));
  vertex((spl[i+3][3]/(scaleIt * 2)) + width/2, (3.0776842 * (spl[i+3][3])/(scaleIt * 2)) + height/2, i - (day * 10));
  vertex(((spl[i+4][3]/(scaleIt * 2)) * -1) + width/2, (3.0776842 * (spl[i+4][3]/(scaleIt * 2))) + height/2, i - (day * 10));
  vertex(((spl[i+5][3]/scaleIt) * -1) + width/2, (0.7265424 * (spl[i+5][3]/scaleIt)) + height/2, i - (day * 10));
  vertex(((spl[i+6][3]/scaleIt) * -1) + width/2, height/2, i - (day * 10));
  vertex(((spl[i+7][3]/scaleIt) * -1) + width/2, (-0.7265424 * (spl[i+7][3]/scaleIt)) + height/2, i - (day * 10));
  vertex(((spl[i+8][3]/(scaleIt * 2)) * -1) + width/2, (-3.0776842 * (spl[i+8][3]/(scaleIt * 2))) + height/2, i - (day * 10));
  vertex((spl[i+9][3]/(scaleIt * 2)) + width/2, (-3.0776842 * (spl[i+9][3]/(scaleIt * 2))) + height/2, i - (day * 10));
  endShape(CLOSE);
  }
  
  strokeWeight(1);
  for (int i = 1; i < spl.length - 20; i = i + 10){
  if (((i + 10) - (day * 10)) != 1){
      noFill();
      stroke(352, 73, 83, 100);
  } else {
      noStroke();
      fill(352, 73, 83, 180);
  }
  //draw difference
  beginShape();  
  vertex((percentDiff(spl[i][3], spl[i + 10][3])) + width/2, (-0.72654253 * (percentDiff(spl[i][3], spl[i + 10][3]))) + height/2, (i + 10) - (day * 10) -1);
  vertex((percentDiff(spl[i+1][3], spl[(i+1) + 10][3])) + width/2, height/2, (i + 10) - (day * 10) -1);
  vertex((percentDiff(spl[i+2][3], spl[(i+2) + 10][3])) + width/2, (0.7265424 * (percentDiff(spl[i+2][3], spl[(i+2) + 10][3]))) + height/2, (i + 10) - (day * 10) -1);
  vertex((percentDiff(spl[i+3][3], spl[(i+3) + 10][3]) / 2) + width/2, (3.0776842 * (percentDiff(spl[i+3][3], spl[(i+3) + 10][3])) / 2) + height/2, (i + 10) - (day * 10) -1);
  vertex(((percentDiff(spl[i+4][3], spl[(i+4) + 10][3]) / 2) * -1) + width/2, (3.0776842 * (percentDiff(spl[i+4][3], spl[(i+4) + 10][3])) / 2) + height/2, (i + 10) - (day * 10) -1);
  vertex(((percentDiff(spl[i+5][3], spl[(i+5) + 10][3])) * -1) + width/2, (0.7265424 * (percentDiff(spl[i+5][3], spl[(i+5) + 10][3]))) + height/2, (i + 10) - (day * 10) -1);
  vertex(((percentDiff(spl[i+6][3], spl[(i+6) + 10][3])) * -1) + width/2, height/2, (i + 10) - (day * 10) -1);
  vertex(((percentDiff(spl[i+7][3], spl[(i+7) + 10][3])) * -1) + width/2, (-0.7265424 * (percentDiff(spl[i+7][3], spl[(i+7) + 10][3]))) + height/2, (i + 10) - (day * 10) -1);
  vertex(((percentDiff(spl[i+8][3], spl[(i+8) + 10][3]) / 2) * -1) + width/2, (-3.0776842 * (percentDiff(spl[i+8][3], spl[(i+8) + 10][3])) / 2) + height/2, (i + 10) - (day * 10) -1);
  vertex((percentDiff(spl[i+9][3], spl[(i+9) + 10][3]) / 2) + width/2, (-3.0776842 * (percentDiff(spl[i+9][3], spl[(i+9) + 10][3])) / 2) + height/2, (i + 10) - (day * 10) -1);
  endShape(CLOSE); 
  }
  
  displayControls();
}

float percentDiff(float a, float b) {
  //if a is larger than b, return ((a - b)/a* 100.0).  Otherwise return ((b - a)/b* 100.0) 
  float diff = (a >= b) ? ((a - b)/a * 100.0) : ((b - a)/b * 100.0);
  return diff;
}

void displayControls() {
  currCameraMatrix = new PMatrix3D(g3.camera);
  camera();
  controlP5.draw();
  g3.camera = currCameraMatrix;
}

void addToRadioButton(RadioButton theRadioButton, String theName, int theValue ) {
  Toggle t = theRadioButton.addItem(theName,theValue);
  t.captionLabel().setColorBackground(color(30, 30, 32));
  t.captionLabel().style().movePadding(2,0,-1,2);
  t.captionLabel().style().moveMargin(-2,0,0,-3);
  t.captionLabel().style().backgroundWidth = 46;
}

void controlEvent(ControlEvent theEvent) {
  //print("got an event from "+theEvent.group().name()+"\t");
  for(int i = 0; i < 1; i++) {
    //print(int(theEvent.group().arrayValue()[i]));
  }
  //println("\t "+theEvent.group().value());
  whichYear = (int)theEvent.group().value();
}

class MyControlListener implements ControlListener {
  public void controlEvent(ControlEvent theEvent) {
    cam.setMouseControlled(false);
  }
}


